<?php
/******************************************************************************/
//                                                                            //
//                             InstantCMS v2.00                               //
//                        http://www.instantcms.ru/                           //
//                                                                            //
//                   written by InstantCMS Team, 2007-2013                    //
//                produced by InstantSoft, (www.instantsoft.ru)               //
//                                                                            //
//                        LICENSED BY GNU/GPL v2                              //
//                                                                            //
/******************************************************************************/
// by WebMan v.5 (04.05.2014)

if(!defined('VALID_RUN')) { die('ACCESS DENIED'); }

class cmsDebug {

	private static $instance;
	
	public $config;	// Текущая рабочая конфигурация
	
	//----------------------------------------------------------------------------
	// Массив с информацией об отладке, доступный снаружи через get()
	// (время - в округлённых милисекундах)
	// Актуализируется на текущий момент из локальных переменных вызовом debugCalc()
	private $data = array(
		'full_time' 				=> 0,	// Полное время выполнения (скрипт + отладка)
		'script_time' 			=> 0,	// Время выполнения скрипта (без учёта времени отладки)
		'debug_time' 				=> 0,	// Суммарное время, потраченное модулем отладки

		'php_time' 					=> 0,	// Суммарное время выполнения php в скрипте (без SQL)
	
		'point_count'				=> 0,	// Количество контрольных точек

		'sql_time' 					=> 0,	// Суммарное время всех подключений и запросов к БД
		'sql_time_connect' 	=> 0,	// Суммарное время всех подключений к БД
		'sql_time_query' 		=> 0,	// Суммарное время всех запросов
		'sql_count_connect' => 0,	// Количество подключений к БД
		'sql_count_query' 	=> 0,	// Количество запросов к БД

		'auto_time' 				=> 0,	// Суммарное время всех автозагрузок классов
		'auto_count'				=> 0,	// Количество автозагрузок классов

		'include_time' 			=> 0,	// Суммарное время всех инклудов
		'include_count'			=> 0,	// Количество инклудов

		'cache_time' 				=> 0,	// Суммарное время всех обращений к кэшу
		'cache_time_set' 		=> 0,	// Суммарное время всех обращений SET к кэшу
		'cache_time_get' 		=> 0,	// Суммарное время всех обращений GET к кэшу
		'cache_count_set' 	=> 0,	// Количество обращений SET к кэшу
		'cache_count_get' 	=> 0,	// Количество обращений GET к кэшу

		'memory_start' 			=> 0,	// Начальное значение памяти скрипта
		'memory_end' 				=> 0,	// Конечное значение памяти скрипта
		'memory_used' 			=> 0,	// Использование памяти скриптом (memory_end - memory_start)
		'memory_peak' 			=> 0	// Максимальное использование памяти скриптом
	);

	//----------------------------------------------------------------------------
	// Переменные
	private $time_start 			 = 0;	// Время старта скрипта
	private $debug_times_start = 0;	// Время старта участка кода отладки
	private $debug_times 			 = 0;	// Суммарное время, потраченное модулем отладки, в секундах

	private $point_time_last					= 0;	// Время предыдущей контрольной точки
	private $point_sql_time_last			= 0;	// Время коннектов и запросов к БД в предыдущей контрольной точке
	private $point_sql_count_last			= 0;	// Количество коннектов и запросов к БД в предыдущей контрольной точке
	private $point_include_time_last	= 0;	// Время автозагрузок и инклудов в предыдущей контрольной точке
	private $point_include_count_last	= 0;	// Количество автозагрузок и инклудов в предыдущей контрольной точке
	private $point_memory_last 				= 0;	// Значение памяти в предыдущей контрольной точке
	private $point_debug_on_sql				= FALSE;	// Временно включить показ коннектов и запросов к БД после этой контрольной точки
	private $point_debug_on_include		= FALSE;	// Временно включить показ автозагрузок и инклудов после этой контрольной точки

	private $sql_times_start 		= 0;	// Время старта запроса SQL, пересчитывается для каждого запроса
	private $sql_times_connect		= 0;	// Суммарное время всех подключений к БД в секундах
	private $sql_times_query 		= 0;	// Суммарное время всех запросов в секундах

	private $auto_times_start 		= 0;	// Время старта автозагрузки ядра и классов, пересчитывается для каждой загрузки
	private $auto_times					= 0;	// Суммарное время всех автозагрузок в секундах

	private $include_times_start = 0;	// Время старта инклуда, пересчитывается для каждого инклуда
	private $include_times 			= 0;	// Суммарное время всех инклудов в секундах

	private $cache_times_start 	= 0;	// Время старта обращения к кэшу, пересчитывается при каждом обращении
	private $cache_times_set 		= 0;	// Суммарное время всех обращений SET к кэшу в секундах
	private $cache_times_get 		= 0;	// Суммарное время всех обращений GET к кэшу в секундах

	private $debug_html = "";	// html-строка с отладочной инфой
	
	private $includes = array();	// Массив инклудов

// =============================================================================

	public function __construct($params="", $start_time=0) {
		if ($start_time > 0) {
			$this->time_start = $start_time;
		} else {
			$this->time_start = microtime(TRUE);
		}
		$this->point_time_last = $this->time_start;
		
		$this->config = cmsDebugConfig::GetInstance($params);

		$this->data['memory_start'] = $this->memoryRound(memory_get_usage($this->config->memory_real_usage));
		$this->point_memory_last = $this->data['memory_start'];
		
		$this->debug_times = microtime(TRUE) - $this->time_start;
	}

	public function __clone() {}

	public static function getInstance($params="", $start_time=0) {
		if (self::$instance === null) self::$instance = new self($params, $start_time);
		return self::$instance;
	}

	public function __get($name) {
			return (isset($this->data[$name]) ? $this->data[$name] : FALSE );
	}

	public static function get($key){
			return self::getInstance()->$key;
	}

// =============================================================================

	// ---------------------------------------------------------------------------
	/**
	* Запоминает время начала соединения к БД
	* Вставляется непосредственно перед началом соединения
	*/
	public function beginConnect() {
		$this->sql_times_start = microtime(TRUE);
		return true;
	}

	// ---------------------------------------------------------------------------
	/**
	* Подсчитывает и сохраняет замеры runtime соединений с БД
	* Вставляется непосредственно после всех операторов соединения
	* @param str $params Строка параметров соединения для лога, выводится без изменений
	*/
	public function endConnect($params="") {
		$this->debug_times_start = microtime(TRUE);
		
		// Сохраняем runtime
		$rt = $this->debug_times_start - $this->sql_times_start;
		$this->sql_times_connect += $rt;
		$this->data['sql_time_connect'] = $this->timeRound($this->sql_times_connect);
		$this->data['sql_time'] = $this->data['sql_time_connect'] + $this->data['sql_time_query'];
		$this->data['sql_count_connect']++;
		
		// Выход, если отладка SQL запрещена
		if (!$this->config->debug_on_sql && !$this->point_debug_on_sql) {
			$this->debug_times += microtime(TRUE) - $this->debug_times_start;
			return true;
		}
		
		// Выход, если включены фильтры для SQL
		if ($this->config->filter_query_sql <> "" || $this->config->filter_query_from <> "") {
			$this->debug_times += microtime(TRUE) - $this->debug_times_start;
			return true;
		}
		
		// Трассировка вызовов
		$trace = $this->getTrace($this->config->trace_level_sql+1, 1); // Задаём пропуск вызовов из cmsDatabase

		$this->debug_html .= "\n<div class='debug_row'><span class='connect'>Connect ".$this->data['sql_count_connect']."</span> - <strong>".$this->timeRound($rt)."</strong> ms<tt>&nbsp;&nbsp;&nbsp;".$trace['point']."</tt>";
		$this->debug_html .= "<br/><pre class='params'>".$params."</pre>";
		$this->debug_html .= $trace['html']."\n</div>";
		
		$this->debug_times += microtime(TRUE) - $this->debug_times_start;
		return true;
	}

	// ---------------------------------------------------------------------------
	/**
	* Запоминает время начала запроса к БД
	* Вставляется непосредственно перед запросом mysql_query
	*/
	public function beginQuery() {
		$this->sql_times_start = microtime(TRUE);
		return true;
	}

	// ---------------------------------------------------------------------------
	/**
	* Подсчитывает и сохраняет замеры runtime запроса
	* Вставляется непосредственно после запроса mysql_query
	* @param string $sql Строка запроса для лога, выводится без изменений
	* @param array $info Массив с информацией о работе запроса для лога
	*/
	public function endQuery($sql="", $info) {
		$this->debug_times_start = microtime(TRUE);
		
		// Сохраняем runtime
		$rt = $this->debug_times_start - $this->sql_times_start;
		$this->sql_times_query += $rt;
		$this->data['sql_time_query'] = $this->timeRound($this->sql_times_query);
		$this->data['sql_time'] = $this->data['sql_time_connect'] + $this->data['sql_time_query'];
		$this->data['sql_count_query']++;
		
		// Выход, если отладка SQL запрещена
		if (!$this->config->debug_on_sql && !$this->point_debug_on_sql) {
			$this->debug_times += microtime(TRUE) - $this->debug_times_start;
			return true;
		}
		
		// Выход, если не удовлетворяет фильтру по SQL-строке запроса
		if ($this->config->filter_query_sql <> "") {
			if (!mb_stripos($sql, $this->config->filter_query_sql)) {
				$this->debug_times += microtime(TRUE) - $this->debug_times_start;
				return true;
			}
		}
		
		// Трассировка вызовов
		$trace = $this->getTrace($this->config->trace_level_sql+2, 2); // Задаём пропуск вызовов из cmsDatabase
		
		// Выход, если не удовлетворяет фильтру по пути/имени вызывающего файла
		if ($this->config->filter_query_from <> "") {
			if (!mb_stripos($trace['point'], $this->config->filter_query_from)) {
				$this->debug_times += microtime(TRUE) - $this->debug_times_start;
				return true;
			}
		}

		$this->debug_html .= "\n<div class='debug_row'><span class='query'>Query ".$this->data['sql_count_query']."</span> - <strong>".$this->timeRound($rt)."</strong> ms<tt>&nbsp;&nbsp;&nbsp;".$trace['point']."&nbsp;&nbsp;&nbsp;</tt>Affected/returned rows: <strong>".$info['affected_rows']."</strong>&nbsp;&nbsp;&nbsp;".$info['info'];
		if ($info['errno'] > 0) $this->debug_html .= "<br>\n<span class='error'>Error ".$info['errno'].": ".$info['error']."</span>";
		if ($info['warnings']) $this->debug_html .= "<span class='warning'>".$info['warnings']."</span>";
		$this->debug_html .= "<br/><pre class='params'>".nl2br(trim($sql), true)."</pre>";
		$this->debug_html .= $trace['html']."\n</div>";

		$this->debug_times += microtime(TRUE) - $this->debug_times_start;
		return true;
	}

	// ---------------------------------------------------------------------------
	/**
	* Запоминает время начала автозагрузки
	* Вставляется перед инклудом в автозагрузке
	*/
	public function beginAutoload() {
		$this->auto_times_start = microtime(TRUE);
		return true;
	}

	// ---------------------------------------------------------------------------
	/**
	* Подсчитывает и сохраняет замеры runtime автозагрузки
	* Вставляется после инклуда в автозагрузке
	* @param string $class Класс для автозагрузки
	* @param string $file Путь к подключаемому файлу
	* @param void $result Результат попытки автозагрузки
	*/
	public function endAutoload($class="", $file="", $result="") {
		$this->debug_times_start = microtime(TRUE);
		
		// Сохраняем runtime
		$rt = $this->debug_times_start - $this->auto_times_start;
		$this->auto_times += $rt;
		$this->data['auto_time'] = $this->timeRound($this->auto_times);
		$this->data['auto_count']++;
		
		// Выход, если отладка автозагрузки запрещена
		if (!$this->config->debug_on_auto && !$this->point_debug_on_include) {
			$this->debug_times += microtime(TRUE) - $this->debug_times_start;
			return true;
		}
		
		// Трассировка вызовов
		$trace = $this->getTrace($this->config->trace_level_auto+2, 2); // Задаём пропуск вызовов из autoload.php

		$this->debug_html .= "\n<div class='debug_row'><span class='auto'>Autoload ".$this->data['auto_count']."</span> - <strong>".$this->timeRound($rt)."</strong> ms<tt>&nbsp;&nbsp;&nbsp;".$trace['point']."&nbsp;&nbsp;&nbsp;</tt>Result: ".($result==1 ? "<span class='ok'>OK</span>" : "<span class='error'>".$result."</span>");
		$this->debug_html .= "<br/><pre class='params'>".$class."  -  ".($file ? "/".$file : "")."</pre>";
		$this->debug_html .= $trace['html']."\n</div>";

		$this->debug_times += microtime(TRUE) - $this->debug_times_start;
		return true;
	}

	// ---------------------------------------------------------------------------
	/**
	* Запоминает время начала инклуда
	* Вставляется перед инклудом
	*/
	public function beginInclude() {
		$this->include_times_start = microtime(TRUE);
		return true;
	}

	// ---------------------------------------------------------------------------
	/**
	* Подсчитывает и сохраняет замеры runtime инклуда
	* Вставляется после инклуда
	* @param string $file Путь к подключаемому файлу
	* @param void $result Результат попытки инклуда
	*/
	public function endInclude($file="", $result="") {
		$this->debug_times_start = microtime(TRUE);
		
		// Сохраняем runtime
		$rt = $this->debug_times_start - $this->include_times_start;
		$this->include_times += $rt;
		$this->data['include_time'] = $this->timeRound($this->include_times);
		$this->data['include_count']++;

		// Выход, если отладка инклуда запрещена
		if (!$this->config->debug_on_include && !$this->point_debug_on_include) {
			$this->debug_times += microtime(TRUE) - $this->debug_times_start;
			return true;
		}
		
		// Подготавливаем путь
		$root_path = cmsConfig::getInstance()->root_path;
		if (substr($file, 0, strlen($root_path)) == $root_path) $file = substr($file, strlen($root_path));
		
		// Выход, если не удовлетворяет фильтру по пути/имени подключаемого файла
		if ($this->config->filter_include_file <> "") {
			if (!mb_stripos($file, $this->config->filter_include_file)) {
				$this->debug_times += microtime(TRUE) - $this->debug_times_start;
				return true;
			}
		}
		
		// Трассировка вызовов
		$trace = $this->getTrace($this->config->trace_level_include+1, 1); // Задаём пропуск вызовов из includeFile()
		
		// Выход, если не удовлетворяет фильтру по пути/имени подключающего файла
		if ($this->config->filter_include_from <> "") {
			if (!mb_stripos($trace['point'], $this->config->filter_include_from)) {
				$this->debug_times += microtime(TRUE) - $this->debug_times_start;
				return true;
			}
		}
		
		// Сохраняем данные для таблицы количеств инклудов
		if ($file) {
			// Считаем количества
			if (array_key_exists($file, $this->includes)) {
				$this->includes[$file]['count']++;
			} else {
				$this->includes[$file]['count'] = 1;
				$this->includes[$file]['error'] = 0;
			}
			// Считаем ошибки
			$result_bool = ($result == 1 || is_array($result));
			if ($result_bool == FALSE) $this->includes[$file]['error']++;
		}

		$this->debug_html .= "\n<div class='debug_row'><span class='include'>Include ".$this->data['include_count']."</span> - <strong>".$this->timeRound($rt)."</strong> ms<tt>&nbsp;&nbsp;&nbsp;".$trace['point']."&nbsp;&nbsp;&nbsp;</tt>Result: ".($result_bool ? "<span class='ok'>OK</span>" : "<span class='error'>".$result."</span>");
		$this->debug_html .= "<br/><pre class='params'>".(substr($file, 0, 1)!="/" && substr($file, 1, 1)!=":" ? "/" : "").$file."</pre>";
		$this->debug_html .= $trace['html']."\n</div>";

		$this->debug_times += microtime(TRUE) - $this->debug_times_start;
		return true;
	}

	// ---------------------------------------------------------------------------
	/**
	* Запоминает время начала работы с кэшем
	*  Вставляется перед началом работы кэша
	*/
	public function beginCache() {
		$this->cache_times_start = microtime(TRUE);
		return true;
	}

	// ---------------------------------------------------------------------------
	/**
	* Подсчитывает и сохраняет замеры runtime кэша
	* Вставляется по окончании работы кэша
	* @param string $type Тип запроса к кэшу (set, get)
	* @param string $key Ключ элемента кэша
	* @param int $result Результат обращения - размер элемента (для set - в байтах, для get - количество элементов массива)
	*/
	public function endCache($type="", $key="", $result="") {
		$this->debug_times_start = microtime(TRUE);
		
		// Сохраняем runtime
		$rt = $this->debug_times_start - $this->cache_times_start;
		$type=strtolower($type);
		if ($type == "set") {
			$this->cache_times_set += $rt;
			$this->data['cache_time_set'] = $this->timeRound($this->cache_times_set);
			$this->data['cache_count_set']++;
			$result = ($result ? $result : 0)." bytes";
		}
		if ($type == "get") {
			$this->cache_times_get += $rt;
			$this->data['cache_time_get'] = $this->timeRound($this->cache_times_get);
			$this->data['cache_count_get']++;
			if (is_numeric($result)) {
				$result = ($result ? $result : 0)." items";
			} else {
				$result = "<span class='error'>".$result."</span>";
			}
		}
		$this->data['cache_time'] = $this->data['cache_time_set'] + $this->data['cache_time_get'];
		
		// Выход, если отладка кэша запрещена
		if (!$this->config->debug_on_cache) {
			$this->debug_times += microtime(TRUE) - $this->debug_times_start;
			return true;
		}
		
		// Трассировка вызовов
		$trace = $this->getTrace($this->config->trace_level_cache+1, 1); // Задаём пропуск вызовов из cache.php

		$this->debug_html .= "\n<div class='debug_row'><span class='cache_".$type."'>Cache <strong>".strtoupper($type)."</strong> ".($this->data['cache_count_set'] + $this->data['cache_count_get'])."</span> - <strong>".$this->timeRound($rt)."</strong> ms<tt>&nbsp;&nbsp;&nbsp;".$trace['point']."&nbsp;&nbsp;&nbsp;</tt>Result: <span class='ok'>".$result."</span>";
		$this->debug_html .= "<br/><pre class='params'>key: ".$key."</pre>";
		$this->debug_html .= $trace['html']."\n</div>";

		$this->debug_times += microtime(TRUE) - $this->debug_times_start;
		return true;
	}

	// ---------------------------------------------------------------------------
	/**
	* Подсчитывает и сохраняет runtime с момента предыдущей точки или со старта скрипта, если это первая точка
	* Вставляется в нужное место кода строкой вида:
	* if (DEBUG_ON) cmsDebug::getInstance()->checkPoint("NAME","param=".$param, 5, true, true);
	* Все параметры необязательные
	* @param string $name Название точки, выводится в лог в кавычках после "CheckPoint"
	* @param string $params Строка параметров для лога, выводится без изменений
	* @param int $trace_level Глубина трассировки вызовов, 0 - выключить, -1 - по умолчанию из конфига
	* @param bool $sql_debug Временно включить показ SQL с этой контрольной точки
	* @param bool $include_debug Временно включить показ инклудов с этой контрольной точки
	*/
	public function checkPoint($name="", $params="", $trace_level=-1, $sql_debug = FALSE, $include_debug = FALSE) {
		$this->debug_times_start = microtime(TRUE);
		
		$this->data['point_count']++;
		
		// Выход, если показ контрольных точек запрещён
		if (!$this->config->debug_on_point) {
			$this->debug_times += $this->debug_times_start - $this->debug_times_start;
			return true;
		}
		
		// Память и время точки
		$mem = $this->memoryRound(memory_get_usage($this->config->memory_real_usage));
		$rt = $this->debug_times_start - $this->point_time_last;
		$this->point_time_last = $this->debug_times_start;
		$mem_delta = round($mem - $this->point_memory_last, 3) * 1000;
		$this->point_memory_last = $mem;
		
		// Количество и время SQL
		$sql_count = $this->data['sql_count_connect'] + $this->data['sql_count_query'] - $this->point_sql_count_last;
		if ($sql_count > 0) {
			$this->point_sql_count_last = $this->data['sql_count_connect'] + $this->data['sql_count_query'];
			$sql_time = $this->sql_times_connect + $this->sql_times_query - $this->point_sql_time_last;
			$this->point_sql_time_last = $this->sql_times_connect + $this->sql_times_query;
		}
		
		// Количество и время автозагрузок и инклудов
		$include_count = $this->data['auto_count'] + $this->data['include_count'] - $this->point_include_count_last;
		if ($include_count > 0) {
			$this->point_include_count_last = $this->data['auto_count'] + $this->data['include_count'];
			$include_time = $this->auto_times + $this->include_times - $this->point_include_time_last;
			$this->point_include_time_last = $this->auto_times + $this->include_times;
		}
		
		// Трассировка вызовов
		if ($trace_level==-1) $trace_level = $this->config->trace_level_point;
		$trace = $this->getTrace($trace_level+1, 1);

		$this->debug_html .= "\n<div class='debug_row'><span class='point'>Point ".$this->data['point_count'].($name ? "&nbsp;&nbsp;'<strong><tt>".$name."</tt></strong>'" : "")."</span> - ";
		$this->debug_html .= "<strong>".$this->timeRound($rt)."</strong> ms - ";
		$this->debug_html .= "[ ".($sql_count==0 ? 0 : "<strong>".$sql_count."</strong>")." sql : ".($sql_count==0 ? 0 : "<strong>".$this->timeRound($sql_time)."</strong>")." ms ]";
		$this->debug_html .= "[ ".($include_count==0 ? 0 : "<strong>".$include_count."</strong>")." incl : ".($include_count==0 ? 0 : "<strong>".$this->timeRound($include_time)."</strong>")." ms ]";
		$this->debug_html .= "[ ".($mem_delta==0 ? "0" : "<strong>".($mem_delta>0 ? "+" : "").$mem_delta."</strong> KB")." : <strong>".$mem."</strong> MB ]";
		$this->debug_html .= "<tt>&nbsp;&nbsp;&nbsp;".$trace['point']."</tt>";
		$this->debug_html .= ($params ? "<br/><pre class='params'>".print_r($params, TRUE)."</pre>" : "");
		$this->debug_html .= $trace['html']."\n</div>";

		$this->point_debug_on_sql = $sql_debug; // Сохраняем признак временной отладки SQL
		$this->point_debug_on_include = $include_debug; // Сохраняем признак временной отладки инклудов

		$this->debug_times += microtime(TRUE) - $this->debug_times_start;
		return true;
	}

	// ---------------------------------------------------------------------------
	/**
	* Обратная трассировка вызовов функций
	* @param int $trace_level
	* @param int $shift
	* @return array 
	*   'point' Строка с местом нахождения сработавшей точки отладки CheckPoint
	*   'html' Строка html со списком вызывавших функций
	*/
	private function getTrace($trace_level=10, $shift=0) {
		$this->debug_times_start = microtime(TRUE);
		
		$html = "";
		$point = "";
		
		$bt=debug_backtrace(); 
		$lvl = -1;
		$sp = 1; // Начальный сдвиг трассировки вправо
		foreach($bt as $k=>$v) { 
			$lvl++;
			if ($lvl<=$shift) continue;
			if ($lvl>$trace_level+1) break;
			extract($v);
			if (!isset($file)) $file = "";
			if (!isset($line)) $line = "";

			if ($file == "" && $line == "" && $function == "spl_autoload_call") {
				$shift++; // Сдвиг на следующий уровень для пропуска spl_autoload_call без файла
				continue;
			}

/*			if ($shift && strstr($file, "database.php")) {
				$shift++; // Сдвиг на следующий уровень для пропуска cmsDatabase
				continue;
			}
*/
			$file = str_replace($_SERVER['DOCUMENT_ROOT'], "", str_replace("\\", "/", $file));
			if ($lvl==1+$shift) {	// Сработавшая точка отладки
				if ($function == "DebugCheckPoint") $function = "";
				$point = $file." (".$line.") ";
				if ($function) $point .= (isset($class) ? $class.$type : "").$function;
			} else {	// Остальные вызвавшие функции
				$html .= "<br/>".str_repeat("&nbsp;&nbsp;", $sp++);
				$html .= $file." (".$line.") ".(isset($class) ? $class.$type : "").$function;
			}
		}
		
		$ret['point'] = "<span class='file'>".$point."</span>";
		$ret['html'] = "<span class='trace'>".$html."</span>";

		$this->debug_times += microtime(TRUE) - $this->debug_times_start;
		return $ret;
	}

	// ---------------------------------------------------------------------------
	/**
	* Вычисляет оставшиеся значения runTime
	*/
	public function debugCalc() {
		$now = microtime(TRUE);
		
		$this->data['memory_end']  = $this->memoryRound(memory_get_usage($this->config->memory_real_usage));
		$this->data['memory_peak'] = $this->memoryRound(memory_get_peak_usage($this->config->memory_real_usage));
		$this->data['memory_used'] = $this->data['memory_end'] - $this->data['memory_start'];

		$this->data['full_time']   = $this->timeRound($now - $this->time_start);
		$this->data['debug_time']  = $this->timeRound($this->debug_times);
		$this->data['debug_time1'] = $this->data['debug_time'];
		$this->data['script_time'] = $this->data['full_time'] - $this->data['debug_time'];
		$this->data['php_time']		 = $this->data['script_time'] - $this->data['sql_time'] - $this->data['cache_time'];		
	}

	// ---------------------------------------------------------------------------
	/**
	* Возвращает html-строку с итоговой таблицей и подробной информацией по отладке
	* @return string
	*/
	public function getHtmlFull() {
		return $this->getHtmlTable() . $this->getHtmlFilters() . $this->getHtmlIncludes() . $this->getHtmlDebug();
	}

	// ---------------------------------------------------------------------------
	/**
	* Возвращает html-строку с подробной информацией по отладке
	* (по контрольным точкам, запросам, инклудам и т.д.)
	* @return string
	*/
	public function getHtmlDebug() {
		if ($this->debug_html) {
			return "<div id='debug'>".$this->debug_html."</div>";
		} else {
			return "";
		}
	}

	// ---------------------------------------------------------------------------
	/**
	* Возвращает готовую html-таблицу с итоговыми результатами замеров времени выполнения
	* @return string
	*/
	public function getHtmlTable() {
		// Вычисляем значения runTime
		$this->debugCalc();
		
		$config = cmsConfig::getInstance();
		
		// --- Table header ---
		$html = "<div id='debug'><table class='debug'><thead><tr><td><u>Script</u><td><u>PHP</u></td><td><u>SQL</u></td><td><u>Cache</u> - ".($config->cache_enabled ? "<span class='cache_set'>on '".$config->cache_method."'</span>" : "off")."</td><td><u>Memory</u>".($this->config->memory_real_usage ? " (real)" : " (exact)")."</td></tr></thead><tbody><tr valign='top'>";

		// --- Script ---
		$html .= "<td>script time: <font size='+1'><span class='help' title='Script runtime = full time - debug time'><strong>".$this->data['script_time']."</strong></span></font> ms";
		// Emulate lag
		if ($this->config->emulate_lag>0) $html .= "<span class='warning'><br/>emulate lag: <strong>".$this->config->emulate_lag."</strong> ms</span>";
		// Debug time
		$html .= "<span class='debug_info'><br/><br/>debug time: <span class='help' title='Class Debug runtime'><strong>".$this->data['debug_time']."</strong></span> ms</span>";
		// Full time
		$html .= "<span class='debug_info'><br/>full time: <span class='help' title='Full runtime, total time of all script and debug parts'><strong>".$this->data['full_time']."</strong></span> ms</span>";
		// Points count
		$html .= ($this->data['point_count']>0 ? "<span class='point'><br/><br/>check points: <strong>".$this->data['point_count']."</strong></span>" : "");
		$html .= "</td>";

		// --- PHP ---
		$html .= "<td>php time: <strong> ".$this->data['php_time']." </strong> ms <span class='help' title='Percent from script time'>(".$this->percent($this->data['script_time'], $this->data['php_time'])."%)</span><br/>";
		// Autoloads
		$html .= "<span class='auto'>autoloads:<br/>- count: <strong>".$this->data['auto_count']."</strong><br/>- time: <strong>".$this->data['auto_time']."</strong> ms <span class='help' title='Percent from php time'>(".$this->percent($this->data['php_time'], $this->data['auto_time'])."%)</span></span><br/>";
		// Includes
		$html .= "<span class='include'>includes:<br/>- count:	<strong>".$this->data['include_count']."</strong><br/>- time: <strong>".$this->data['include_time']."</strong> ms <span class='help' title='Percent from php time'>(".$this->percent($this->data['php_time'], $this->data['include_time'])."%)</span></span></td>";

		// --- SQL ---
		$html .= "<td>sql time: <strong>".$this->data['sql_time']."</strong> ms <span class='help' title='Percent from script time'>(".$this->percent($this->data['script_time'], $this->data['sql_time'])."%)</span><br/>";
		// Connects
		$html .= "<span class='connect'>connects:<br/>- count: <strong>".$this->data['sql_count_connect']."</strong><br/>- time: <strong>".$this->data['sql_time_connect']."</strong> ms <span class='help' title='Percent from sql time'>(".$this->percent($this->data['sql_time'], $this->data['sql_time_connect'])."%)</span></span><br/>";
		// Queries
		$html .= "<span class='query'>queries:<br/>- count:	<strong>".$this->data['sql_count_query']."</strong><br/>- time: <strong>".$this->data['sql_time_query']."</strong> ms <span class='help' title='Percent from sql time'>(".$this->percent($this->data['sql_time'], $this->data['sql_time_query'])."%)</span></span></td>";

		// --- Cache ---
		$html .= "<td>cache time <strong>".$this->data['cache_time']."</strong> ms <span class='help' title='Percent from script time'>(".$this->percent($this->data['script_time'], $this->data['cache_time'])."%)</span><br/>";
		// SET
		$html .= "<span class='cache_set'>set:<br/>- count: <strong>".$this->data['cache_count_set']."</strong><br/>- time: <strong>".$this->data['cache_time_set']."</strong> ms <span class='help' title='Percent from cache time'>(".$this->percent($this->data['cache_time'], $this->data['cache_time_set'])."%)</span></span><br/>";
		// GET
		$html .= "<span class='cache_get'>get:<br/>- count: <strong>".$this->data['cache_count_get']."</strong><br/>- time: <strong>".$this->data['cache_time_get']."</strong> ms <span class='help' title='Percent from cache time'>(".$this->percent($this->data['cache_time'], $this->data['cache_time_get'])."%)</span><br/></span></td>";

		// --- Memory ---
		$html .= "<td>peak: <span class='help' title='Maximum of memory usage'><strong>".$this->data['memory_peak']."</strong></span> MB<br/>start: <span class='help' title='Memory at script start'>".$this->data['memory_start']."</span> MB<br/>end: <span class='help' title='Memory at script end'>".$this->data['memory_end']."</span> MB<br/>used: <span class='help' title='Used memory = end - start'>".$this->data['memory_used']."</span> MB</td>";
		
		$html .= "</tr></tbody></table></div>";
		return $html;
	}

	// ---------------------------------------------------------------------------
	/**
	* Возвращает html-блок с описанием включённых фильтров
	* @return string
	*/
	public function getHtmlFilters() {

		$html = "";

		// Фильтр по строке SQL по части строки запроса
		if ($this->config->debug_on_sql && $this->config->filter_query_sql <> "") {
			$html .= "<tr><td>SQL string:</td><td><tt>'<span class='query'>".$this->config->filter_query_sql."</span>'</tt></td></tr>";
		}

		// Фильтр по строке SQL
		if ($this->config->debug_on_sql && $this->config->filter_query_from <> "") {
			$html .= "<tr><td>SQL from:</td><td><tt>'<span class='query'>".$this->config->filter_query_from."</span>'</tt></td></tr>";
		}
		
		// Фильтр по подключаемому файлу
		if ($this->config->debug_on_include && $this->config->filter_include_file <> "") {
			$html .= "<tr><td>Included file:</td><td><tt>'<span class='include'>".$this->config->filter_include_file."</span>'</tt></td></tr>";
		}
		
		// Фильтр по подключающему файлу
		if ($this->config->debug_on_include && $this->config->filter_include_from <> "") {
			if ($html <> "") $html .= "<br>";
			$html .= "<tr><td>Included from:</td><td><tt>'<span class='include'>".$this->config->filter_include_from."</span>'</tt></td></tr>";
		}

		if ($html <> "") $html = "<div id='debug'><table class='filter'><thead><tr><td><u>Used filters</u></td></tr></thead><tbody>".$html."</tbody></table></div>";
		
		return $html;
	}

	// ---------------------------------------------------------------------------
	/**
	* Возвращает готовую html-таблицу с количествами инклудов файлов
	* @return string
	*/
	public function getHtmlIncludes() {

		if (count($this->includes) == 0) return "";
		
		$html = "";
		
		$lines = 0;

		ksort($this->includes, SORT_STRING);
		
		foreach($this->includes as $f => $v) {
			if (!$this->config->includes_table_error || $v['count'] > 1 || $v['error'] > 0) {
				$html .= "<tr><td>".$f."</td><td".($v['count']>1 ? " class='warning'" : "")." align='center'>".$v['count']."</td><td align='center'".($v['error']>0 ? " class='error'>".$v['error'] : ">")."</td></tr>";
				$lines++;
			}
		}
		
		$html = "<div id='debug'><table class='debug'><thead><tr><td><u>Included files</u> (".$lines." from ".count($this->includes).")".($this->config->filter_include_file.$this->config->filter_include_from <> "" ? " - filter" : "")."</td><td><u>Count</u></td><td><u>Errors</u></td></tr></thead><tbody>".$html;
		$html .= "</tbody></table></div>";
		return $html;
	}

	// ---------------------------------------------------------------------------
	/**
	* Возвращает признак разрешения показывать отладочную информацию
	* @return bool Разрешено показывать отладку или нет
	*/
	public function allowShow() {
		return (!$this->config->debug_admin_only || cmsUser::getInstance()->is_admin);
	}

	// ---------------------------------------------------------------------------
	/**
	* Возвращает округлённое до милисекунд значение времени $time
	* с количеством знаков после запятой $rnd
	* @param float $time Время в секундах
	* @param int $rnd Требуемое количество знаков после запятой
	* @return float Время в милисекундах
	*/
	private function timeRound($time, $rnd = 1) {
		return round($time*1000, $rnd);
	}
	
	// ---------------------------------------------------------------------------
	/**
	* Возвращает округлённое до мегабайт значение памяти $mem в байтах
	* с количеством знаков после запятой $rnd
	* @param float $mem Значение памяти в байтах
	* @param int $rnd Требуемое количество знаков после запятой
	* @return float Значение памяти в мегабайтах
	*/
	private function memoryRound($mem, $rnd = 3) {
		return round($mem/1048576, $rnd);
	}
	
	// ---------------------------------------------------------------------------
	/**
	* Возвращает процент $part по отношению к $full
	* с количеством знаков после запятой $rnd
	* @param float $full Значение, принимаемое за 100%
	* @param float $part Значение, для которого вычисляется процент
	* @param int $rnd Требуемое количество знаков после запятой
	* @return float Расчётный процент (при $full=0 возвращается 0)
	*/
	private function percent($full, $part, $rnd = 0) {
		return ($full==0 ? 0 : round($part/$full * 100, $rnd));
	}

// =============================================================================

}
?>